@@ -1,13 +1,15 @@
 package CPAN::Changes::Cwalitee;
 
-our $DATE = '2019-07-03'; # DATE
-our $VERSION = '0.001'; # VERSION
+our $DATE = '2019-07-06'; # DATE
+our $VERSION = '0.002'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
 use warnings;
 use Log::ger;
 
+use Cwalitee::Common;
+
 use Exporter qw(import);
 our @EXPORT_OK = qw(
                        calc_cpan_changes_cwalitee
@@ -19,136 +21,58 @@ our %SPEC;
 $SPEC{list_cpan_changes_cwalitee_indicators} = {
     v => 1.1,
     args => {
-        detail => {
-            schema => 'bool*',
-            cmdline_aliases=>{l=>{}},
-        },
-        # XXX filter by severity
-        # XXX filter by module
-        # XXX filter by status
+        %Cwalitee::Common::args_list,
     },
 };
 sub list_cpan_changes_cwalitee_indicators {
-    require PERLANCAR::Module::List;
-
     my %args = @_;
 
-    my @res;
-
-    my $mods = PERLANCAR::Module::List::list_modules(
-        'CPAN::Changes::Cwalitee::', {list_modules=>1, recurse=>1});
-    for my $mod (sort keys %$mods) {
-        (my $mod_pm = "$mod.pm") =~ s!::!/!g;
-        require $mod_pm;
-        my $spec = \%{"$mod\::SPEC"};
-        for my $func (sort keys %$spec) {
-            next unless $func =~ /\Aindicator_/;
-            my $funcmeta = $spec->{$func};
-            (my $name = $func) =~ s/\Aindicator_//;
-            my $rec = {
-                name => $name,
-                summary  => $funcmeta->{summary},
-                priority => $funcmeta->{'x.indicator.priority'} // 50,
-                severity => $funcmeta->{'x.indicator.severity'} // 3,
-                status   => $funcmeta->{'x.indicator.status'} // 'stable',
-            };
-            if ($args{_return_coderef}) {
-                $rec->{code} = \&{"$mod\::$func"};
-            }
-            push @res, $rec;
-        }
-    }
-
-    unless ($args{detail}) {
-        @res = map { $_->{name} } @res;
-    }
-
-    [200, "OK", \@res];
+    Cwalitee::Common::list_cwalitee_indicators(
+        prefix => 'CPAN::Changes::',
+        %args,
+    );
 }
 
 $SPEC{calc_cpan_changes_cwalitee} = {
     v => 1.1,
     args => {
+        %Cwalitee::Common::args_calc,
         path => {
             schema => 'pathname*',
             req => 1,
-            pos => 0,
         },
     },
 };
 sub calc_cpan_changes_cwalitee {
     require File::Slurper;
 
-    my %args = @_;
+    my %fargs = @_;
+    my $path = delete $fargs{path};
 
-    my $res = list_cpan_changes_cwalitee_indicators(
-        detail => 1,
-        _return_coderef => 1,
-    );
-    return $res unless $res->[0] == 200;
-
-    my @res;
-    my $r = {
-        path => $args{path},
-        file_content => File::Slurper::read_text($args{path}),
-        abstract => $args{abstract},
-    };
-    my $num_run = 0;
-    my $num_success = 0;
-    my $num_fail = 0;
     my $parse_attempted;
-    for my $ind (sort {
-        $a->{priority} <=> $b->{priority} ||
-            $a->{name} cmp $b->{name}
-        } @{ $res->[2] }) {
-
-        if ($ind->{priority} > 1 && !$parse_attempted++) {
-            require CPAN::Changes::Subclass::Cwalitee;
-            eval {
-                $r->{parsed} = CPAN::Changes::Subclass::Cwalitee->load_string(
-                    $r->{file_content});
+    Cwalitee::Common::calc_cwalitee(
+        prefix => 'CPAN::Changes::',
+        %fargs,
+        code_init_r => sub {
+            return {
+                path => $path,
+                file_content => File::Slurper::read_text($path),
             };
-        }
+        },
+        code_fixup_r => sub {
+            my %cargs = @_;
+            my $ind = $cargs{indicator};
+            my $r   = $cargs{r};
 
-        my $indres = $ind->{code}->(r => $r);
-        $num_run++;
-        my ($result, $result_summary);
-        if ($indres->[0] == 200) {
-            if ($indres->[2]) {
-                $result = 0;
-                $num_fail++;
-                $result_summary = $indres->[2];
-            } else {
-                $result = 1;
-                $num_success++;
-                $result_summary = '';
+            if ($ind->{priority} > 1 && !$parse_attempted++) {
+                require CPAN::Changes::Subclass::Cwalitee;
+                eval {
+                    $r->{parsed} = CPAN::Changes::Subclass::Cwalitee->load_string(
+                        $r->{file_content});
+                };
             }
-        } elsif ($indres->[0] == 412) {
-            $result = undef;
-            $result_summary = "Cannot be run".($indres->[1] ? ": $indres->[1]" : "");
-        } else {
-            return [500, "Unexpected result when checking indicator ".
-                        "'$ind->{name}': $indres->[0] - $indres->[1]"];
-        }
-        my $res = {
-            num => $num_run,
-            indicator => $ind->{name},
-            priority => $ind->{priority},
-            severity => $ind->{severity},
-            summary  => $ind->{summary},
-            result => $result,
-            result_summary => $result_summary,
-        };
-        push @res, $res;
-    }
-
-    push @res, {
-        indicator      => 'Score',
-        result         => sprintf("%.2f", $num_run ? ($num_success / $num_run)*100 : 0),
-        result_summary => "$num_success out of $num_run",
-    };
-
-    [200, "OK", \@res];
+        },
+    );
 }
 
 1;
@@ -166,7 +90,7 @@ CPAN::Changes::Cwalitee - Calculate the cwalitee of your CPAN Changes file
 
 =head1 VERSION
 
-This document describes version 0.001 of CPAN::Changes::Cwalitee (from Perl distribution CPAN-Changes-Cwalitee), released on 2019-07-03.
+This document describes version 0.002 of CPAN::Changes::Cwalitee (from Perl distribution CPAN-Changes-Cwalitee), released on 2019-07-06.
 
 =head1 SYNOPSIS
 
@@ -181,8 +105,6 @@ This document describes version 0.001 of CPAN::Changes::Cwalitee (from Perl dist
 
 =head1 DESCRIPTION
 
-B<EARLY RELEASE. CURRENTLY ONLY CONTAINS A MINIMUM SET OF INDICATORS.>
-
 B<What is CPAN Changes cwalitee?> A metric to attempt to gauge the quality of
 your CPAN Changes file. Since actual quality is hard to measure, this metric is
 called a "cwalitee" instead. The cwalitee concept follows "kwalitee" [1] which
@@ -192,6 +114,11 @@ term "cwalitee" does not refer to a specific, particular subject. There can be
 "CPAN Changes cwalitee" (which is handled by this module), "module abstract
 cwalitee", and so on.
 
+=head1 INTERNAL NOTES
+
+B<Indicator priority.> At priority 10, Changes file is parsed using
+CPAN::Changes and the result # it put in 'parsed' key.
+
 =head1 FUNCTIONS
 
 
@@ -207,6 +134,34 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<exclude_indicator> => I<array[str]>
+
+Do not use these indicators.
+
+=item * B<exclude_indicator_module> => I<array[perl::modname]>
+
+Do not use indicators from these modules.
+
+=item * B<exclude_indicator_status> => I<array[str]>
+
+Do not use indicators having these statuses.
+
+=item * B<include_indicator> => I<array[str]>
+
+Only use these indicators.
+
+=item * B<include_indicator_module> => I<array[perl::modname]>
+
+Only use indicators from these modules.
+
+=item * B<include_indicator_status> => I<array[str]> (default: ["stable"])
+
+Only use indicators having these statuses.
+
+=item * B<min_indicator_severity> => I<uint> (default: 1)
+
+Minimum indicator severity.
+
 =item * B<path>* => I<pathname>
 
 =back
@@ -238,6 +193,38 @@ Arguments ('*' denotes required arguments):
 
 =item * B<detail> => I<bool>
 
+=item * B<exclude> => I<array[str]>
+
+Exclude by name.
+
+=item * B<exclude_module> => I<array[perl::modname]>
+
+Exclude by module.
+
+=item * B<exclude_status> => I<array[str]>
+
+Exclude by status.
+
+=item * B<include> => I<array[str]>
+
+Include by name.
+
+=item * B<include_module> => I<array[perl::modname]>
+
+Include by module.
+
+=item * B<include_status> => I<array[str]> (default: ["stable"])
+
+Include by status.
+
+=item * B<max_severity> => I<uint> (default: 5)
+
+Maximum severity.
+
+=item * B<min_severity> => I<uint> (default: 1)
+
+Minimum severity.
+
 =back
 
 Returns an enveloped result (an array).
